@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2014 The pygit2 contributors
+ * Copyright 2010-2015 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -70,6 +70,21 @@ int_to_loose_object_type(int type_id)
     }
 }
 
+PyObject *
+wrap_repository(git_repository *c_repo)
+{
+    Repository *py_repo = PyObject_GC_New(Repository, &RepositoryType);
+
+    if (py_repo) {
+        py_repo->repo = c_repo;
+        py_repo->config = NULL;
+        py_repo->index = NULL;
+        py_repo->owned = 1;
+    }
+
+    return (PyObject *)py_repo;
+}
+
 int
 Repository_init(Repository *self, PyObject *args, PyObject *kwds)
 {
@@ -334,6 +349,26 @@ Repository_lookup_branch(Repository *self, PyObject *args)
 }
 
 
+PyDoc_STRVAR(Repository_path_is_ignored__doc__,
+  "Check if a path is ignored in the repository.");
+
+PyObject *
+Repository_path_is_ignored(Repository *self, PyObject *args)
+{
+    int ignored;
+    char *path;
+
+    if (!PyArg_ParseTuple(args, "s", &path))
+        return NULL;
+
+    git_ignore_path_is_ignored(&ignored, self->repo, path);
+    if (ignored == 1)
+        Py_RETURN_TRUE;
+
+    Py_RETURN_FALSE;
+}
+
+
 PyDoc_STRVAR(Repository_revparse_single__doc__,
   "revparse_single(revision) -> Object\n"
   "\n"
@@ -526,7 +561,8 @@ Repository_workdir__set__(Repository *self, PyObject *py_workdir)
 PyDoc_STRVAR(Repository_merge_base__doc__,
   "merge_base(oid, oid) -> Oid\n"
   "\n"
-  "Find as good common ancestors as possible for a merge.");
+  "Find as good common ancestors as possible for a merge.\n"
+  "Returns None if there is no merge base between the commits");
 
 PyObject *
 Repository_merge_base(Repository *self, PyObject *args)
@@ -550,6 +586,10 @@ Repository_merge_base(Repository *self, PyObject *args)
         return NULL;
 
     err = git_merge_base(&oid, self->repo, &oid1, &oid2);
+
+    if (err == GIT_ENOTFOUND)
+        Py_RETURN_NONE;
+
     if (err < 0)
         return Error_set(err);
 
@@ -622,7 +662,7 @@ Repository_merge(Repository *self, PyObject *py_oid)
     if (err < 0)
         return Error_set(err);
 
-    checkout_opts.checkout_strategy = GIT_CHECKOUT_SAFE_CREATE;
+    checkout_opts.checkout_strategy = GIT_CHECKOUT_SAFE | GIT_CHECKOUT_RECREATE_MISSING;
     err = git_merge(self->repo,
                     (const git_annotated_commit **)&commit, 1,
                     &merge_opts, &checkout_opts);
@@ -634,6 +674,46 @@ Repository_merge(Repository *self, PyObject *py_oid)
     Py_RETURN_NONE;
 }
 
+PyDoc_STRVAR(Repository_cherrypick__doc__,
+  "cherrypick(id)\n"
+  "\n"
+  "Cherry-pick the given oid, producing changes in the index and working directory.\n"
+  "\n"
+  "Merges the given commit into HEAD as a cherrypick, writing the results into the\n"
+  "working directory. Any changes are staged for commit and any conflicts\n"
+  "are written to the index. Callers should inspect the repository's\n"
+  "index after this completes, resolve any conflicts and prepare a\n"
+  "commit.");
+
+PyObject *
+Repository_cherrypick(Repository *self, PyObject *py_oid)
+{
+    git_commit *commit;
+    git_oid oid;
+    int err;
+    size_t len;
+    git_cherrypick_options cherrypick_opts = GIT_CHERRYPICK_OPTIONS_INIT;
+
+    len = py_oid_to_git_oid(py_oid, &oid);
+    if (len == 0)
+        return NULL;
+
+    err = git_commit_lookup(&commit, self->repo, &oid);
+    if (err < 0)
+        return Error_set(err);
+
+    cherrypick_opts.checkout_opts.checkout_strategy = GIT_CHECKOUT_SAFE;
+    err = git_cherrypick(self->repo,
+                    commit,
+                    (const git_cherrypick_options *)&cherrypick_opts);
+
+    git_commit_free(commit);
+    if (err < 0)
+        return Error_set(err);
+
+    Py_RETURN_NONE;
+}
+
 PyDoc_STRVAR(Repository_walk__doc__,
   "walk(oid[, sort_mode]) -> iterator\n"
   "\n"
@@ -782,8 +862,70 @@ Repository_create_blob_fromdisk(Repository *self, PyObject *args)
 }
 
 
+PyDoc_STRVAR(Repository_create_blob_fromiobase__doc__,
+    "create_blob_fromiobase(io.IOBase) -> Oid\n"
+    "\n"
+    "Create a new blob from an IOBase object.");
+
+
+int read_chunk(char *content, size_t max_length, void *payload)
+{
+    PyObject   *py_file;
+    PyObject   *py_bytes;
+    char       *bytes;
+    Py_ssize_t  size;
+
+    py_file  = (PyObject *)payload;
+    py_bytes = PyObject_CallMethod(py_file, "read", "i", max_length);
+    if (!py_bytes)
+        return -1;
+
+    size = 0;
+    if (py_bytes != Py_None) {
+        bytes = PyBytes_AsString(py_bytes);
+        size  = PyBytes_Size(py_bytes);
+        memcpy(content, bytes, size);
+    }
+
+    Py_DECREF(py_bytes);
+    return size;
+}
+
+PyObject *
+Repository_create_blob_fromiobase(Repository *self, PyObject *py_file)
+{
+    git_oid   oid;
+    PyObject *py_is_readable;
+    int       is_readable;
+    int       err;
+
+    py_is_readable = PyObject_CallMethod(py_file, "readable", NULL);
+    if (!py_is_readable) {
+        if (PyErr_ExceptionMatches(PyExc_AttributeError))
+          PyErr_SetObject(PyExc_TypeError, py_file);
+        return NULL;
+    }
+
+    is_readable = PyObject_IsTrue(py_is_readable);
+    Py_DECREF(py_is_readable);
+
+    if (!is_readable) {
+        Py_DECREF(py_file);
+        PyErr_SetString(PyExc_TypeError, "expected readable IO type");
+        return NULL;
+    }
+
+    err = git_blob_create_fromchunks(&oid, self->repo, NULL, &read_chunk,
+                                     py_file);
+    if (err < 0)
+        return Error_set(err);
+
+    return git_oid_to_python(&oid);
+}
+
+
 PyDoc_STRVAR(Repository_create_commit__doc__,
-  "create_commit(reference, author, committer, message, tree, parents[, encoding]) -> Oid\n"
+  "create_commit(reference_name, author, committer, message, tree, parents[, encoding]) -> Oid\n"
   "\n"
   "Create a new commit object, return its oid.");
 
@@ -934,7 +1076,7 @@ Repository_create_branch(Repository *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "sO!|i", &c_name, &CommitType, &py_commit, &force))
         return NULL;
 
-    err = git_branch_create(&c_reference, self->repo, c_name, py_commit->commit, force, NULL, NULL);
+    err = git_branch_create(&c_reference, self->repo, c_name, py_commit->commit, force);
     if (err < 0)
         return Error_set(err);
 
@@ -981,6 +1123,56 @@ out:
 }
 
 
+PyDoc_STRVAR(Repository_listall_reference_objects__doc__,
+  "listall_reference_objects() -> [Reference, ...]\n"
+  "\n"
+  "Return a list with all the reference objects in the repository.");
+
+PyObject *
+Repository_listall_reference_objects(Repository *self, PyObject *args)
+{
+    git_reference_iterator *iter;
+    git_reference *ref = NULL;
+    int err;
+    PyObject *list;
+
+    list = PyList_New(0);
+    if (list == NULL)
+        return NULL;
+
+    if ((err = git_reference_iterator_new(&iter, self->repo)) < 0)
+        return Error_set(err);
+
+    while ((err = git_reference_next(&ref, iter)) == 0) {
+        PyObject *py_ref = wrap_reference(ref, self);
+        if (py_ref == NULL)
+            goto error;
+
+        err = PyList_Append(list, py_ref);
+        Py_DECREF(py_ref);
+
+        if (err < 0)
+            goto error;
+    }
+
+    git_reference_iterator_free(iter);
+    if (err == GIT_ITEROVER)
+        err = 0;
+
+    if (err < 0) {
+        Py_CLEAR(list);
+        return Error_set(err);
+    }
+
+    return list;
+
+error:
+    git_reference_iterator_free(iter);
+    Py_CLEAR(list);
+    return NULL;
+}
+
+
 PyDoc_STRVAR(Repository_listall_branches__doc__,
   "listall_branches([flag]) -> [str, ...]\n"
   "\n"
@@ -1044,6 +1236,38 @@ error:
     return NULL;
 }
 
+PyDoc_STRVAR(Repository_listall_submodules__doc__,
+  "listall_submodules() -> [str, ...]\n"
+  "\n"
+  "Return a list with all submodule paths in the repository.\n");
+
+static int foreach_path_cb(git_submodule *submodule, const char *name, void *payload)
+{
+    PyObject *list = (PyObject *)payload;
+    PyObject *path = to_unicode(git_submodule_path(submodule), NULL, NULL);
+
+    return PyList_Append(list, path);
+}
+
+PyObject *
+Repository_listall_submodules(Repository *self, PyObject *args)
+{
+    int err;
+    PyObject *list;
+
+    list = PyList_New(0);
+    if (list == NULL)
+        return NULL;
+
+    err = git_submodule_foreach(self->repo, foreach_path_cb, list);
+    if (err != 0) {
+        Py_DECREF(list);
+        return Py_None;
+    }
+
+    return list;
+}
+
 
 PyDoc_STRVAR(Repository_lookup_reference__doc__,
   "lookup_reference(name) -> Reference\n"
@@ -1107,7 +1331,7 @@ Repository_create_reference_direct(Repository *self,  PyObject *args,
     if (err < 0)
         return NULL;
 
-    err = git_reference_create(&c_reference, self->repo, c_name, &oid, force, NULL, NULL);
+    err = git_reference_create(&c_reference, self->repo, c_name, &oid, force, NULL);
     if (err < 0)
         return Error_set(err);
 
@@ -1141,7 +1365,7 @@ Repository_create_reference_symbolic(Repository *self,  PyObject *args,
         return NULL;
 
     err = git_reference_symbolic_create(&c_reference, self->repo, c_name,
-                                        c_target, force, NULL, NULL);
+                                        c_target, force, NULL);
     if (err < 0)
         return Error_set(err);
 
@@ -1401,8 +1625,8 @@ PyDoc_STRVAR(Repository_reset__doc__,
     "\n"
     "Resets current head to the provided oid.\n"
     "reset_type:\n"
-    "GIT_RESET_SOFT: resets head to point to oid, but does not modfy working copy, and leaves the changes in the index.\n"
-    "GIT_RESET_MIXED: resets head to point to oid, but does not modfy working copy. It empties the index too.\n"
+    "GIT_RESET_SOFT: resets head to point to oid, but does not modify working copy, and leaves the changes in the index.\n"
+    "GIT_RESET_MIXED: resets head to point to oid, but does not modify working copy. It empties the index too.\n"
     "GIT_RESET_HARD: resets head to point to oid, and resets too the working copy and the content of the index.\n");
 
 PyObject *
@@ -1426,17 +1650,36 @@ Repository_reset(Repository *self, PyObject* args)
 
     err = git_object_lookup_prefix(&target, self->repo, &oid, len,
                                    GIT_OBJ_ANY);
-    err = err < 0 ? err : git_reset(self->repo, target, reset_type, NULL, NULL, NULL);
+    err = err < 0 ? err : git_reset(self->repo, target, reset_type, NULL);
     git_object_free(target);
     if (err < 0)
         return Error_set_oid(err, &oid, len);
     Py_RETURN_NONE;
 }
 
+PyDoc_STRVAR(Repository_expand_id__doc__,
+    "expand_id(hex) -> Oid\n"
+    "\n"
+    "Expand a string into a full Oid according to the objects in this repsitory.\n");
+
+PyObject *
+Repository_expand_id(Repository *self, PyObject *py_hex)
+{
+    git_oid oid;
+    int err;
+
+    err = py_oid_to_git_oid_expand(self->repo, py_hex, &oid);
+    if (err < 0)
+        return NULL;
+
+    return git_oid_to_python(&oid);
+}
+
 PyMethodDef Repository_methods[] = {
     METHOD(Repository, create_blob, METH_VARARGS),
     METHOD(Repository, create_blob_fromworkdir, METH_VARARGS),
     METHOD(Repository, create_blob_fromdisk, METH_VARARGS),
+    METHOD(Repository, create_blob_fromiobase, METH_O),
     METHOD(Repository, create_commit, METH_VARARGS),
     METHOD(Repository, create_tag, METH_VARARGS),
     METHOD(Repository, TreeBuilder, METH_VARARGS),
@@ -1444,11 +1687,14 @@ PyMethodDef Repository_methods[] = {
     METHOD(Repository, merge_base, METH_VARARGS),
     METHOD(Repository, merge_analysis, METH_O),
     METHOD(Repository, merge, METH_O),
+    METHOD(Repository, cherrypick, METH_O),
     METHOD(Repository, read, METH_O),
     METHOD(Repository, write, METH_VARARGS),
     METHOD(Repository, create_reference_direct, METH_VARARGS),
     METHOD(Repository, create_reference_symbolic, METH_VARARGS),
     METHOD(Repository, listall_references, METH_NOARGS),
+    METHOD(Repository, listall_reference_objects, METH_NOARGS),
+    METHOD(Repository, listall_submodules, METH_NOARGS),
     METHOD(Repository, lookup_reference, METH_O),
     METHOD(Repository, revparse_single, METH_O),
     METHOD(Repository, status, METH_NOARGS),
@@ -1458,9 +1704,11 @@ PyMethodDef Repository_methods[] = {
     METHOD(Repository, lookup_note, METH_VARARGS),
     METHOD(Repository, git_object_lookup_prefix, METH_O),
     METHOD(Repository, lookup_branch, METH_VARARGS),
+    METHOD(Repository, path_is_ignored, METH_VARARGS),
     METHOD(Repository, listall_branches, METH_VARARGS),
     METHOD(Repository, create_branch, METH_VARARGS),
     METHOD(Repository, reset, METH_VARARGS),
+    METHOD(Repository, expand_id, METH_O),
     METHOD(Repository, _from_c, METH_VARARGS),
     METHOD(Repository, _disown, METH_NOARGS),
     {NULL}
