@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2010-2014 The pygit2 contributors
+# Copyright 2010-2015 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2,
@@ -39,6 +39,14 @@ import os
 from os.path import join, realpath
 import sys
 
+import six
+
+if six.PY2:
+    from urllib import pathname2url
+    
+if six.PY3:
+    from urllib.request import pathname2url
+
 # Import from pygit2
 from pygit2 import GIT_OBJ_ANY, GIT_OBJ_BLOB, GIT_OBJ_COMMIT
 from pygit2 import init_repository, clone_repository, discover_repository
@@ -156,6 +164,11 @@ class RepositoryTest(utils.BareRepoTestCase):
             commit.message)
         self.assertRaises(ValueError, self.repo.__getitem__, too_short_prefix)
 
+    def test_expand_id(self):
+        commit_sha = '5fe808e8953c12735680c257f56600cb0de44b10'
+        expanded = self.repo.expand_id(commit_sha[:7])
+        self.assertEqual(commit_sha, expanded.hex)
+
     @unittest.skipIf(__pypy__ is not None, "skip refcounts checks in pypy")
     def test_lookup_commit_refcount(self):
         start = sys.getrefcount(self.repo)
@@ -185,14 +198,47 @@ class RepositoryTest(utils.BareRepoTestCase):
 
     def test_hashfile(self):
         data = "bazbarfoo"
-        tempfile_path = tempfile.mkstemp()[1]
-        with open(tempfile_path, 'w') as fh:
+        handle, tempfile_path = tempfile.mkstemp()
+        with os.fdopen(handle, 'w') as fh:
             fh.write(data)
         hashed_sha1 = hashfile(tempfile_path)
         os.unlink(tempfile_path)
         written_sha1 = self.repo.create_blob(data)
         self.assertEqual(hashed_sha1, written_sha1)
 
+    def test_conflicts_in_bare_repository(self):
+        def create_conflict_file(repo, branch, content):
+            oid = repo.create_blob(content.encode('utf-8'))
+            tb = repo.TreeBuilder()
+            tb.insert('conflict', oid, pygit2.GIT_FILEMODE_BLOB)
+            tree = tb.write()
+
+            sig = pygit2.Signature('Author', 'author@example.com')
+            commit = repo.create_commit(branch.name, sig, sig,
+                    'Conflict', tree, [branch.target])
+            self.assertIsNotNone(commit)
+            return commit
+
+        b1 = self.repo.create_branch('b1', self.repo.head.peel())
+        c1 = create_conflict_file(self.repo, b1, 'ASCII - abc')
+        b2 = self.repo.create_branch('b2', self.repo.head.peel())
+        c2 = create_conflict_file(self.repo, b2, 'Unicode - äüö')
+
+        index = self.repo.merge_commits(c1, c2)
+        self.assertIsNotNone(index.conflicts)
+
+        # ConflictCollection does not allow calling len(...) on it directly so
+        # we have to calculate length by iterating over its entries
+        self.assertEqual(sum(1 for _ in index.conflicts), 1)
+
+        (a, t, o) = index.conflicts['conflict']
+        diff = self.repo.merge_file_from_index(a, t, o)
+        self.assertEqual(diff, '''<<<<<<< conflict
+ASCII - abc
+=======
+Unicode - äüö
+>>>>>>> conflict
+''')
 
 class RepositoryTest_II(utils.RepoTestCase):
 
@@ -295,6 +341,24 @@ class RepositoryTest_II(utils.RepoTestCase):
         self.assertEqual(commit.hex,
                          'acecd5ea2924a4b900e7e149496e1f4b57976e51')
 
+        # Create a commit without any merge base to any other
+        sig = pygit2.Signature("me", "me@example.com")
+        indep = self.repo.create_commit(None, sig, sig, "a new root commit",
+                                        self.repo[commit].peel(pygit2.Tree).id, [])
+
+        self.assertEqual(None, self.repo.merge_base(indep, commit))
+
+    def test_ahead_behind(self):
+        ahead, behind = self.repo.ahead_behind('5ebeeebb320790caf276b9fc8b24546d63316533',
+                                               '4ec4389a8068641da2d6578db0419484972284c8')
+        self.assertEqual(1, ahead)
+        self.assertEqual(2, behind)
+
+        ahead, behind = self.repo.ahead_behind('4ec4389a8068641da2d6578db0419484972284c8',
+                                               '5ebeeebb320790caf276b9fc8b24546d63316533')
+        self.assertEqual(2, ahead)
+        self.assertEqual(1, behind)
+
     def test_reset_hard(self):
         ref = "5ebeeebb320790caf276b9fc8b24546d63316533"
         with open(os.path.join(self.repo.workdir, "hello.txt")) as f:
@@ -426,6 +490,18 @@ class EmptyRepositoryTest(utils.EmptyRepoTestCase):
         self.assertFalse(self.repo.head_is_detached)
 
 
+class StringTypesRepositoryTest(utils.NoRepoTestCase):
+
+    def test_bytes_string(self):
+        repo_path = b'./test/data/testrepo.git/'
+        pygit2.Repository(repo_path)
+
+    def test_unicode_string(self):
+        # String is unicode because of unicode_literals
+        repo_path = './test/data/testrepo.git/'
+        pygit2.Repository(repo_path)
+
+
 class CloneRepositoryTest(utils.NoRepoTestCase):
 
     def test_clone_repository(self):
@@ -443,7 +519,12 @@ class CloneRepositoryTest(utils.NoRepoTestCase):
     def test_clone_repository_and_remote_callbacks(self):
         src_repo_relpath = "./test/data/testrepo.git/"
         repo_path = os.path.join(self._temp_dir, "clone-into")
-        url = 'file://' + os.path.realpath(src_repo_relpath)
+        url = pathname2url(os.path.realpath(src_repo_relpath))
+        
+        if url.startswith('///'):
+            url = 'file:' + url
+        else:
+            url = 'file://' + url
 
         def create_repository(path, bare):
             return init_repository(path, bare)
@@ -458,10 +539,9 @@ class CloneRepositoryTest(utils.NoRepoTestCase):
         self.assertIsNotNone(repo.remotes["custom_remote"])
 
     def test_clone_with_credentials(self):
-        credentials = pygit2.UserPass("libgit2", "libgit2")
         repo = clone_repository(
             "https://bitbucket.org/libgit2/testgitrepository.git",
-            self._temp_dir, credentials=credentials)
+            self._temp_dir, callbacks=pygit2.RemoteCallbacks(credentials=pygit2.UserPass("libgit2", "libgit2")))
 
         self.assertFalse(repo.is_empty)
 
